# encoding: utf8
import re
from decimal import Decimal, InvalidOperation


class ValidationError(Exception):

    def __init__(self, validator):
        self.validator = validator.__class__.__name__
        self.message = validator.message


class Validator(object):

    def __init__(self, message=None):
        if message:
            self.message = message

    def setField(self, field):
        self.field = field

    def __call__(self, value):
        if not self.validate(value):
            raise ValidationError(self)


class NotEmpty(Validator):
    message = u"This element is mandatory."

    def validate(self, value):
        if value == None:
            return False
        elif type(value) == str and value.strip() == '':
            return False
        elif type(value) == bytes and value.strip() == b'':
            return False
        elif type(value) in [list, dict, tuple] and len(value) == 0:
            return False
        return True


class IsDigit(Validator):
    message = u'This element must be a digit.'

    def validate(self, value):
        if re.search('^-{0,1}[0-9]+$', value):
            return True
        else:
            return False


class IsDecimal(Validator):
    message = u'This element must be a decimal.'

    def validate(self, value):
        try:
            Decimal(value)
            return True
        except InvalidOperation:
            return False


class Email(Validator):
    message = u"Email is not valid."

    def validate(self, value):
        if len(value) > 7:
            if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", value) != None:
                return True
        return False



from formskit.tests.base import FormskitTestCase
from formskit import Field, Button
from formskit.validators import NotEmpty, IsDigit


class FieldTest(FormskitTestCase):

    def test_creation(self):
        name = 'name'
        label = 'label'
        validators = [None, 1]
        field = Field(name, validators, label)

        self.assertEqual(name, field.name)
        self.assertEqual(label, field.label)
        self.assertEqual(validators, field.validators)
        self.assertEqual(None, field.value)
        self.assertEqual(None, field.form)
        self.assertEqual(None, field.message)
        self.assertFalse(field.ignore)

        field = Field(name, validators, label, True)

        self.assertEqual(name, field.name)
        self.assertEqual(label, field.label)
        self.assertEqual(validators, field.validators)
        self.assertEqual(None, field.value)
        self.assertEqual(None, field.form)
        self.assertEqual(None, field.message)
        self.assertTrue(field.ignore)


    def test_init(self):
        form = 123
        field = Field(None, None, None)
        field.initForm(form)

        self.assertEqual(form, field.form)

    def test_validate(self):
        empty = NotEmpty()
        field = Field('name', [empty,])

        self.assertEqual(False, field.validate())
        self.assertEqual(field, empty.field)

        field.value = 'green'

        self.assertEqual(True, field.validate())
        self.assertEqual(field, empty.field)

    def test_two_validators(self):
        field = Field('name', [NotEmpty(), IsDigit()])

        self.assertEqual(False, field.validate())

        field.value = 'green'

        self.assertEqual(False, field.validate())

        field.value = '15'

        self.assertEqual(True, field.validate())

    def test_buttons(self):
        label = 'button test'
        name = 'button name'
        field = Button(name, label)
        self.assertTrue(field.ignore)
        self.assertEqual(name, field.name)
        self.assertEqual(label, field.label)



# encoding: utf8
class FormValidationError(Exception):

    def __init__(self, validator):
        self.validator = validator.__class__.__name__
        self.message = validator.message


class FormValidator(object):

    message = None

    def setForm(self, form):
        self.form = form

    def __call__(self):
        if not self.validate():
            raise FormValidationError(self)


class MustBeTheSame(FormValidator):

    message_tpl = '%s must be the same!'

    def __init__(self, names, label):
        self.names = names
        self.label = label

    @property
    def message(self):
        return self.message_tpl % (self.label,)

    def validate(self):
        values = []
        for name in self.names:
            for field in self.form.fields[name]:
                values.append(field.value)
        first = values.pop(0)
        for value in values:
            if first != value:
                return False
        return True



import logging
import unittest
from .base import alltests


def import_tests():
    from . import validators
    from . import field
    from . import form
    from . import formvalidators
    from . import errors


def get_all_test_suite():
    def start_logging():
        logging.basicConfig(level=logging.INFO, format="%(asctime)-15s:%(message)s", filename='test.log')
        logging.getLogger('formskit').info('\n\t*** TESTING STARTED ***')

    def create_test_suit():
        suite = unittest.TestLoader()
        prepered_all_test_cases = []
        for test_case in alltests:
            prepered_all_test_cases.append(
                suite.loadTestsFromTestCase(test_case)
            )
        return unittest.TestSuite(prepered_all_test_cases)

    start_logging()
    import_tests()
    return create_test_suit()



from formskit.tests.base import FormskitTestCase
from formskit.field import Field
import formskit.validators as VAL


class ValidatorTest(FormskitTestCase):
    cls = None
    good_samples = None
    bad_samples = None

    def test_create(self):
        sample_text = 'something'
        validator = self.cls(sample_text)

        self.assertEqual(sample_text, validator.message)

    def test_success(self):
        validator = self.cls('')

        for sample in self.good_samples:
            self.assertNone(validator(sample))

    def test_fail(self):
        validator = self.cls('')

        for sample in self.bad_samples:
            self.assertRaises(VAL.ValidationError, validator, sample)

    def test_setForm(self):
        field = Field('my field')
        validator = self.cls('')
        validator.setField(field)

        self.assertEqual(field, validator.field)


class NotEmptyValidatorTest(ValidatorTest):
    cls = VAL.NotEmpty

    good_samples = [
        'z', '0', '12312312dasd213123', ',', ' ad sda ',
    ]

    bad_samples = [
        ' ', '', None, [], {}, b'',
    ]


class IsDigitValidatorTest(ValidatorTest):
    cls = VAL.IsDigit

    good_samples = [
        '1', '123123123', '012312', '-123123',
    ]

    bad_samples = [
        'a', '', '-123213.12323', '2a',
    ]

class EmailValidatorTest(ValidatorTest):
    cls = VAL.Email

    good_samples = [
        'msocek@gmail.com',
        '1asd@asdasd.sadad.pl',
    ]

    bad_samples = [
        'a', '', '-123213.12323', '2a',
        ' ', '@asdweq.asdad.pl', '1asd@asdasd.sadad.asdpl',
    ]

class IsDecimalTest(ValidatorTest):
    cls = VAL.IsDecimal

    good_samples = [
        '1',
        '1.2',
        '1.2213123123123123',
        '123123123.123123'
        '234234234234',
        '-123123.123123',
    ]

    bad_samples = [
        'a', '', '-123213,12323', '2a',
        ' ', '@asdweq.asdad.pl', '1asd@asdasd.sadad.asdpl',
    ]



from mock import MagicMock

from formskit import Form, Field, Button
from formskit.errors import BadValue
from formskit.tests.base import FormskitTestCase
from formskit.validators import NotEmpty


class SampleObject(object):

    def __init__(self, name1='value1', name2='value2'):
        self.name1 = name1
        self.name2 = name2


class SampleObject2(object):

    def __init__(self, name1='value1'):
        self.name1 = name1


class Form1(Form):

    submitTest = False
    overal = False
    name1 = 'name1'

    def submit(self, data):
        self._test_data = data
        self.submitTest = True

    def overalValidation(self, data):
        return self.overal

    def createForm(self):
        self.addField(Field(self.name1))


class Form2(Form):
    name1 = 'name1'
    name2 = 'name2'

    def createForm(self):
        self.addField(Field(self.name1, [NotEmpty()]))
        self.addField(Field(self.name2, [NotEmpty()]))
        self.addField(Button('button', label=u'Zaloguj'))


class Form3(Form):
    name1 = 'name1'
    name2 = 'name2'

    def createForm(self):
        self.addField(Field(self.name1, label=u'my label'))
        self.addField(Field(self.name2, [NotEmpty()]))
        self.addField(Button('button', label=u'Zaloguj'))


class FormTest(FormskitTestCase):

    def test_get_label(self):
        form = Form3()
        self.assertEqual('my label', form.get_label('name1'))

    def test_get_error(self):
        form = Form3()
        form.fields['something'] = [MagicMock()]
        form.fields['something'][0].error = 'error'
        self.assertEqual('error', form.get_error('something'))

    def test_get_error_fail(self):
        form = Form3()
        self.assertEqual(False, form.get_error('something'))

    def test_get_message(self):
        form = Form3()
        form.fields['something'] = [MagicMock()]
        form.fields['something'][0].message = 'message'
        self.assertEqual('message', form.get_message('something'))

    def test_get_message_fail(self):
        form = Form3()
        self.assertEqual(None, form.get_message('something'))

    def test_get_value(self):
        form = Form3()
        form.fields['something'] = [MagicMock()]
        form.fields['something'][0].value = 'value'
        self.assertEqual('value', form.get_value('something'))

    def test_get_value_fail(self):
        form = Form3()
        self.assertEqual(None, form.get_value('something'))

    def test_name(self):
        self.assertEqual(Form1().name, 'Form1')

    def test_addField(self):
        name = 'rosomak'
        field = Field(name)
        form = Form1()
        form.addField(field)

        self.assertTrue(name in form.field_patterns)
        self.assertEqual(field, form.field_patterns[name])
        self.assertEqual(form, field.form)

    def test_gatherDataFromFields(self):
        name1 = 'rosomak'
        value1 = 'wolverine'
        name2 = 'stefan'
        value2 = 'robin'

        field1 = Field(name1)
        field2 = Field(name2)

        form = Form1()
        form.addField(field1)
        form.addField(field2)
        form._assign_field_value(name1, value1)
        form._assign_field_value(name2, value2)

        data = form.gatherDataFromFields()
        self.assertTrue(name1 in data)
        self.assertTrue(name2 in data)
        self.assertEqual([value1, ], data[name1])
        self.assertEqual([value2, ], data[name2])

    def test_createForm(self):
        form = Form2()

        self.assertTrue(form.name1 in form.field_patterns)
        self.assertTrue(form.name2 in form.field_patterns)

    def test_isThisFormSubmited(self):
        form = Form2()

        self.assertFalse(form._isThisFormSubmited({}))
        self.assertFalse(
            form._isThisFormSubmited({'form_name': ['Formasd2', ]}))
        self.assertTrue(form._isThisFormSubmited({'form_name': ['Form2', ]}))

    def test_gatherFormsData(self):
        form = Form1()

        name1 = 'name1'
        name2 = 'name2'

        field1 = Field(name1)
        field2 = Field(name2)
        form.addField(field1)
        form.addField(field2)

        value1 = ['value1', ]
        value2 = ['value2', ]

        data = {
            'form_name': 'Form1',
            name1: value1,
        }

        data[name2] = value2

        form._gatherFormsData(data)

        self.assertEqual(value1[0], form.fields[name1][0].value)
        self.assertEqual(value2[0], form.fields[name2][0].value)

        data['name3'] = 'value3'
        self.assertRaises(BadValue, form._gatherFormsData, data)

    def test_validateFields(self):
        form = Form3()
        form._assign_field_value(form.name1, '')
        form._assign_field_value(form.name2, '')

        self.assertFalse(form._validateFields())
        self.assertFalse(form.fields['name1'][0].error)
        self.assertTrue(form.fields['name2'][0].error)
        self.assertNone(form.fields['name1'][0].message)
        self.assertEqual(NotEmpty.message, form.fields['name2'][0].message)

        form.fields[form.name1][0].value = 'asasdsd'
        self.assertFalse(form._validateFields())
        self.assertFalse(form.fields['name1'][0].error)
        self.assertTrue(form.fields['name2'][0].error)
        self.assertNone(form.fields['name1'][0].message)
        self.assertEqual(NotEmpty.message, form.fields['name2'][0].message)

        form.fields[form.name2][0].value = 'asd'
        self.assertTrue(form._validateFields())
        self.assertFalse(form.fields['name1'][0].error)
        self.assertFalse(form.fields['name2'][0].error)
        self.assertNone(form.fields['name1'][0].message)
        self.assertNone(form.fields['name2'][0].message)

    def test_validate_and_submit(self):
        form = Form1()
        form.overal = False

        self.assertFalse(form._validate_and_submit())
        self.assertFalse(form.submitTest)

        form = Form1()
        form.overal = True
        self.assertTrue(form._validate_and_submit())
        self.assertTrue(form.submitTest)
        self.assertEqual({}, form._test_data)

    def test_call(self):
        form = Form1()

        value1 = ['value1', ]

        data = {
            form.form_name_value: ['bad name', ],
            form.name1: value1,
        }

        self.assertNone(form(data))

        data[form.form_name_value] = [form.name, ]
        self.assertFalse(form(data))
        self.assertFalse(form.submitTest)

        form.overal = True
        self.assertTrue(form(data))
        self.assertTrue(form.submitTest)
        self.assertEqual({form.name1: value1}, form._test_data)

    def test_ignore(self):
        good_name = 'name1'
        good_value = ['value1', ]
        button_name = 'button1'
        form = Form1()
        form.overal = True
        form.addField(Button(button_name, 'label'))
        form.addField(Field(good_name))

        data = {
            form.form_name_value: [form.name, ],
            good_name: good_value,
        }

        self.assertTrue(form(data))
        self.assertEqual(good_value[0], form.fields[good_name][0].value)
        self.assertEqual(good_value, form._test_data[good_name])
        self.assertFalse(button_name in form._test_data)

    def test_assign_field_value(self):
        form = Form1()
        form.fields = {}
        form._assign_field_value('name1', 'value1')

        self.assertEqual(Field, type(form.fields['name1'][0]))
        self.assertEqual(1, len(form.fields['name1']))

    def test_missing_value(self):
        form = Form2()
        result = form({
            'form_name': ['Form2', ],
            'name1': ['one'],
        })
        self.assertTrue(result is False)

    def test_missing_value_2(self):
        form = Form2()
        result = form({
            'form_name': ['Form2', ],
            'name1': ['one'],
            'name2': [],
        })
        self.assertTrue(result is False)

    def test_missing_value_3(self):
        form = Form2()
        result = form({
            'form_name': ['Form2', ],
            'name1': ['one'],
            'name2': [''],
        })
        self.assertTrue(result is False)

    def test_not_missing_value(self):
        form = Form2()
        result = form({
            'form_name': ['Form2', ],
            'name1': ['one'],
            'name2': ['two'],
        })
        self.assertTrue(result is True)


class FormUpdateTest(FormskitTestCase):

    def setUp(self):
        super(FormUpdateTest, self).setUp()
        self.form = Form3()

    def test_simple(self):
        s1 = SampleObject()
        self.form.update(s1)

        self.assertEqual(s1.name1, self.form.fields['name1'][0].value)
        self.assertEqual(s1.name2, self.form.fields['name2'][0].value)

    def test_with_names(self):
        s3 = SampleObject('value1_3', 'value2_3')
        self.form.update(s3, ['name1'])

        self.assertEqual(s3.name1, self.form.fields['name1'][0].value)
        self.assertTrue('name2' not in self.form.fields)

    def test_dict(self):
        s4 = {
            'name1': 'value1_4',
            'name2': 'value2_4',
        }

        self.form.update(s4, method='dict')

        self.assertEqual(s4['name1'], self.form.fields['name1'][0].value)
        self.assertEqual(s4['name2'], self.form.fields['name2'][0].value)

    def test_missing_attribute(self):
        s5 = SampleObject2('value1_5')
        self.assertRaises(AttributeError, self.form.update, s5)

    def test_obj_default_none(self):
        s6 = SampleObject2('value1_6')
        self.form.update(s6, method='obj_default_none')
        self.assertEqual(s6.name1, self.form.fields['name1'][0].value)
        self.assertEqual(None, self.form.fields['name2'][0].value)

    def test_own_get_method(self):
        s7 = SampleObject2('value1_7')

        def method(obj, name):
            return name
        self.form.update(s7, method=method)
        self.assertEqual('name1', self.form.fields['name1'][0].value)
        self.assertEqual('name2', self.form.fields['name2'][0].value)

    def test_ignore_missing(self):
        s8 = SampleObject2('value1_8')
        self.form.update(s8, ignore_missing=True)
        self.assertEqual(s8.name1, self.form.fields['name1'][0].value)
        self.assertTrue('name2' not in self.form.fields)

    def test_update_after_change(self):
        changed_value = 'value3_9'
        s9 = SampleObject('value1_9', 'value2_9')
        self.form._assign_field_value('name1', changed_value)
        self.form.update(s9)

        self.assertEqual(s9.name1, self.form.fields['name1'][0].value)
        self.assertEqual(s9.name2, self.form.fields['name2'][0].value)


class FormInitialDataTest(FormskitTestCase):

    data = {
        'form_name': ['Form2', ],
        'name1': ['one'],
    }

    initial_data = {
        'name2': ['iname2'],
        'name1': ['iname1'],
    }

    def test_simple(self):
        """Should make initial data as default. The form data should override it."""
        form = Form2()
        form(self.data, initial_data=self.initial_data)
        self.assertEqual({
            'name1': ['one', ],
            'name2': ['iname2', ],
        }, form.data)



from formskit.validators import ValidationError


class Field(object):

    def __init__(self, name, validators=None, label=None, ignore=False):
        self.name = name
        self.label = label
        self.validators = validators or []
        self.ignore = ignore
        self.value = None
        self.form = None
        self.message = None
        self.error = False

    def initForm(self, form):
        self.form = form

    def validate(self):
        try:
            for validator in self.validators:
                validator.setField(self)
                validator(self.value)
            self.message = None
            self.error = False
            return True
        except ValidationError as ex:
            self.message = ex.message
            self.error = True
            return False

class Button(Field):
    def __init__(self, name, label):
        super(Button, self).__init__(name, label=label, ignore=True)



from formskit.tests.base import FormskitTestCase
from formskit import errors

class ErrorsTests(FormskitTestCase):

    def test_BadValue(self):
        error = errors.BadValue('name1')
        self.assertEqual('name1', error.name)
        self.assertEqual('name1', str(error))

    def test_ValueNotPresent(self):
        error = errors.ValueNotPresent('name2')
        self.assertEqual('name2', error.name)
        self.assertEqual('name2', str(error))



from formskit import Form, Field
from formskit.tests.base import FormskitTestCase
from formskit.formvalidators import FormValidator, MustBeTheSame

class Form1(Form):

    name1 = 'something'
    name2 = 'something2'

    def createForm(self):
        self.addField(Field(self.name1))
        self.addField(Field(self.name2))

        self.addFormValidator(MustBeTheSame([self.name1, self.name2], 'input'))


class FormValidatorTest(FormskitTestCase):

    def test_base(self):
        validator = FormValidator()
        validator.setForm(self)

        self.assertEqual(self, validator.form)

    def test_MustBeTheSame(self):
        form = Form1()
        form._assign_field_value(form.name1, '1')
        form._assign_field_value(form.name2, '2')

        self.assertFalse(form._validateFields())
        self.assertEqual(True, form.error)
        self.assertEqual('input must be the same!', form.message)

        form = Form1()
        form._assign_field_value(form.name1, '1')
        form._assign_field_value(form.name2, '1')

        self.assertTrue(form._validateFields())
        self.assertEqual(False, form.error)
        self.assertEqual(None, form.message)



from unittest import TestCase
alltests = []
from six import add_metaclass


class FormskitTestCaseType(type):

    def __init__(cls, name, bases, dct):
        super(FormskitTestCaseType, cls).__init__(name, bases, dct)
        if not name in ['FormskitTestCase', 'ValidatorTest']:
            alltests.append(cls)


@add_metaclass(FormskitTestCaseType)
class FormskitTestCase(TestCase):

    def assertNone(self, obj):
        return self.assertEqual(None, obj)



# -*- encoding: utf-8 -*-
from setuptools import setup, find_packages

install_requires = [
    'python-dateutil',
    'six',
]

if __name__ == '__main__':
    setup(name='formskit',
          version='0.4.1',
          author=['Dominik "Socek" Długajczy'],
          author_email=['msocek@gmail.com', ],
          packages=find_packages(),
          test_suite='formskit.tests.get_all_test_suite',
          )



from copy import copy

from formskit.errors import BadValue
from formskit.formvalidators import FormValidationError


class Form(object):

    form_name_value = 'form_name'

    def __init__(self):
        self.fields = {}
        self.field_patterns = {}
        self.formValidators = []
        self.createForm()
        self.error = False
        self.message = None
        self.data = None

    @property
    def name(self):
        return self.__class__.__name__

    def addField(self, field):
        self.field_patterns[field.name] = field
        field.initForm(self)

    def addFormValidator(self, validator):
        validator.setForm(self)
        self.formValidators.append(validator)

    def gatherDataFromFields(self):
        data = {}
        for name, fields in self.fields.items():
            data[name] = []
            if not fields[0].ignore:
                for field in fields:
                    data[name].append(field.value)
        return data

    def _isThisFormSubmited(self, data):
        return self.form_name_value in data and data[self.form_name_value] == [self.name, ]

    def _assign_field_value(self, name, value, index=None):
        if not name in self.fields:
            self.fields[name] = []
        if index is None:
            index = len(self.fields[name])
        try:
            field = self.fields[name][index]
        except IndexError:
            field = copy(self.field_patterns[name])
            self.fields[name].append(field)
        field.value = value

    def _assign_missing_values(self):
        field_names = []
        for name, value in self.field_patterns.items():
            if not value.ignore:
                field_names.append(name)

        for name in field_names:
            if name not in self.fields:
                self._assign_field_value(name, None)

    def _gatherFormsData(self, data):
        data = dict(data)
        data.pop(self.form_name_value, None)

        field_names = list(self.field_patterns)

        for name, value in data.items():
            if name in field_names:
                index = -1
                for small_value in value:
                    index += 1
                    self._assign_field_value(name, small_value, index)
            else:
                raise BadValue(name)

        self._assign_missing_values()

    def _validateFields(self):
        def validateFields():
            validation = True
            for name, fields in self.fields.items():
                if not fields[0].ignore:
                    for field in fields:
                        validation &= field.validate()
            return validation

        def validateGlobalValidators():
            try:
                for formValidator in self.formValidators:
                    formValidator()
            except FormValidationError as er:
                self.message = er.message
                self.error = True
                return False
            return True
        #----------------------------------------------------------------------
        validation = validateFields()
        if validation:
            validation = validateGlobalValidators()
        return validation

    def _validate_and_submit(self):
        data = self.gatherDataFromFields()
        if self._validateFields():
            if self.overalValidation(data):
                self.data = data
                self.submit(data)
                return True
            else:
                self.error = True
        return False

    def __call__(self, data, initial_data={}):
        self.reset(initial_data)
        if self._isThisFormSubmited(data):
            self._gatherFormsData(data)
            return self._validate_and_submit()

    def reset(self, initial_data={}):
        self.fields = {}
        self._gatherFormsData(initial_data)

    def update(self, obj, names=None, method='obj', ignore_missing=False):
        def getattr_obj(obj, name):
            return getattr(obj, name)

        def getattr_dict(obj, name):
            return obj[name]

        def getattr_obj_default_none(obj, name):
            return getattr(obj, name, None)

        def get_method(method):
            getattr_funs = {
                'obj': getattr_obj,
                'dict': getattr_dict,
                'obj_default_none': getattr_obj_default_none,
            }
            if type(method) == str:
                return getattr_funs[method]
            else:
                return method

        def make_names(names):
            if names is None:
                return list(self.field_patterns)
            else:
                return names

        def set_form_field(name, obj, get):
            if not self.field_patterns[name].ignore and self.field_patterns[name].value in [None, '']:
                data = get(obj, name)
                if name in self.fields:
                    self.fields[name][0].value = data
                else:
                    self._assign_field_value(name, data)

        get = get_method(method)
        names = make_names(names)
        if ignore_missing:
            for name in names:
                try:
                    set_form_field(name, obj, get)
                except (AttributeError, KeyError):
                    continue
        else:
            for name in names:
                set_form_field(name, obj, get)

    def overalValidation(self, data):
        return True  # pragma: no cover

    def createForm(self):
        pass  # pragma: no cover

    def submit(self, data):
        pass  # pragma: no cover

    def get_label(self, name):
        return self.field_patterns[name].label

    def get_error(self, name, index=0):
        field = self.get_field(name, index)
        try:
            return field.error
        except AttributeError:
            return False

    def get_message(self, name, index=0):
        field = self.get_field(name, index)
        try:
            return field.message
        except AttributeError:
            return None

    def get_value(self, name, index=0):
        field = self.get_field(name, index)
        try:
            return field.value
        except AttributeError:
            return None

    def get_field(self, name, index=0):
        if name in self.fields:
            fields = self.fields[name]
            if len(fields) > index:
                return fields[index]
        return None



class BadValue(Exception):

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name


class ValueNotPresent(Exception):

    def __init__(self, name):
        self.name = name

    def __str__(self):
        return self.name
